library platforms_source_gen;

import 'dart:io';
import 'dart:convert';

import 'package:platforms_source_gen/method_parse.dart';

import 'android_gen.dart';
import 'class_parse.dart';
import 'property_parse.dart';

class GenClassBean {
  ClassInfo classInfo;
  List<MethodInfo> methods = [];
  List<Property> properties = [];
}

void main() {
  platforms_source_gen_init(
      "./lib/example", //you dart file path
      "com.siyehua.example", //your android's  java class package name
      "./Android_gen" //your android file save path
      );
}

void platforms_source_gen_init(
    String dir, String javaPackage, String androidSavePath) async {
  // String path = "./lib/example";
  // String javaPackage = "com.siyehua.example";
  // String androidSavePath = "./Android_gen";

  Directory directory = Directory(dir);
  directory.listSync().forEach((file) {
    if (file is File) {
      _parseFile(file, javaPackage, androidSavePath);
    }
  });
}

void _parseFile(File file, String javaPackage, String androidSavePath) async {
  List<String> importList = [];
  List<GenClassBean> genClassBeans = [];
  GenClassBean genClassBean = GenClassBean();
  await file
      .openRead()
      .transform(utf8.decoder)
      .transform(new LineSplitter())
      .forEach((str) {
    // print('read line: $str');
    str = _checkLineStr(str);
    if (str.isEmpty) {
    } else if (_foundImport(str, importList)) {
    } else {
      int classResult = parseClass(str);
      ClassInfo classInfo = getClassInfo();
      if (classResult == 1) {
        print("class start:${classInfo.name}");
      } else if (classResult == 0 && classInfo != 0) {
        //check property or method
        _formatLine(str, genClassBean);
      } else if (classResult == 2) {
        print("class end:${classInfo.name}");
        genClassBean.classInfo = classInfo;
        genClassBeans.add(genClassBean);

        //clear tmp class info
        genClassBean = GenClassBean();
        clearClassInfo();
      }
    }
  });
  // print(classMap);
  JavaCreate.create(javaPackage, androidSavePath, genClassBeans);
}

String _checkLineStr(String str) {
  str = str.replaceAll("\t", "").replaceAll("\n", "").trim();
  if (str.startsWith("//") || str.isEmpty) {
    return "";
  }
  return str;
}

bool _foundImport(String str, List<String> importList) {
  if (str.startsWith("import")) {
    importList.add(str);
    return true;
  }
  return false;
}

String tmpLineStr = "";

void _formatLine(String str, GenClassBean genClassBean) {
  MethodInfo methodInfo = parseMethod(str);
  if (methodInfo != null) {
    genClassBean.methods.add(methodInfo);
    return;
  }
  //pattern line end
  RegExp exp = new RegExp(r";[ ]*//|;");
  if (!exp.hasMatch(str)) {
    //the line is't end, so continue...

    tmpLineStr += str;
  } else {
    // print("property str: " + tmpLineStr + str);
    int endIndex = str.lastIndexOf(";");
    str = str.substring(0, endIndex);
    str = tmpLineStr + str;
    str = str.replaceAll("new", "");
    str = str.trimLeft().trimRight().trim();
    genClassBean.properties.add(parseProperty(str));
    tmpLineStr = ""; //clear tmp str
  }
}
